package com.huaweicloud.sdk.iot.amqpdemo.client;

import com.huaweicloud.sdk.iot.amqpdemo.LogUtil;

import java.net.InetAddress;
import java.util.UUID;

public class AmqpClientOptions {
    private String host;

    private int port = 5671;

    private String accessKey;

    private String accessCode;

    private String instanceId;

    /**
     * 客户端ID，建议每个客户端配置具备标示性，方便console关联
     */
    private String clientId = UUID.randomUUID().toString();

    /**
     * true: SDK自动ACK（默认）
     * false:收到消息后，需要手动调用delivery.settle()
     */
    private boolean autoAcknowledge = true;

    /**
     * 重连时延（ms）
     */
    private long reconnectDelay = 3000L;

    /**
     * 最大重连时延（ms）,随着重连次数增加重连时延逐渐增加
     */
    private long maxReconnectDelay = 30 * 1000L;

    /**
     * 最大重连次数,默认值-1，代表没有限制
     */
    private long maxReconnectAttempts = -1;

    /**
     * The idle timeout in milliseconds after which the connection will be failed if the peer sends no AMQP frames. Default is 60000.
     */
    private int idleTimeout = 60 * 1000;

    /**
     * The values below control how many messages the remote peer can send to the client and be held in a pre-fetch buffer for each consumer instance.
     */
    private int queuePrefetch = 1000;

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public int getPort() {
        return port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public String getAccessKey() {
        return accessKey;
    }

    public void setAccessKey(String accessKey) {
        this.accessKey = accessKey;
    }

    public String getAccessCode() {
        return accessCode;
    }

    public void setAccessCode(String accessCode) {
        this.accessCode = accessCode;
    }

    public String getInstanceId() {
        return instanceId;
    }

    public void setInstanceId(String instanceId) {
        this.instanceId = instanceId;
    }

    public String getClientId() {
        return clientId;
    }

    public void setClientId(String clientId) {
        this.clientId = clientId;
    }

    public boolean isAutoAcknowledge() {
        return autoAcknowledge;
    }

    public void setAutoAcknowledge(boolean autoAcknowledge) {
        this.autoAcknowledge = autoAcknowledge;
    }

    public long getReconnectDelay() {
        return reconnectDelay;
    }

    public void setReconnectDelay(long reconnectDelay) {
        this.reconnectDelay = reconnectDelay;
    }

    public long getMaxReconnectDelay() {
        return maxReconnectDelay;
    }

    public void setMaxReconnectDelay(long maxReconnectDelay) {
        this.maxReconnectDelay = maxReconnectDelay;
    }

    public long getMaxReconnectAttempts() {
        return maxReconnectAttempts;
    }

    public void setMaxReconnectAttempts(long maxReconnectAttempts) {
        this.maxReconnectAttempts = maxReconnectAttempts;
    }

    public int getIdleTimeout() {
        return idleTimeout;
    }

    public void setIdleTimeout(int idleTimeout) {
        this.idleTimeout = idleTimeout;
    }

    public int getQueuePrefetch() {
        return queuePrefetch;
    }

    public void setQueuePrefetch(int queuePrefetch) {
        this.queuePrefetch = queuePrefetch;
    }

    public String getEndpointStr() {
        return String.format("%s:%s", host, port);
    }

    public boolean checkParamValid() {
        if (isBlank(host) || isBlank(accessKey) || isBlank(accessCode)) {
            LogUtil.error("Invalid input.");
            return false;
        }
        if (port < 0 || port > 65535) {
            LogUtil.error("Invalid input port.");
            return false;
        }
        try {
            InetAddress.getByName(host);
        } catch (Exception e) {
            LogUtil.exceptionError(e, "Error host input.");
            return false;
        }
        return true;
    }

    private boolean isBlank(String str) {
        return str == null || str.length() == 0;
    }
}
