package com.huaweicloud.sdk.iot.amqpdemo.handler;

import com.huaweicloud.sdk.iot.amqpdemo.LogUtil;

import org.apache.qpid.proton.engine.BaseHandler;
import org.apache.qpid.proton.engine.Event;
import org.apache.qpid.proton.engine.Sasl;
import org.apache.qpid.proton.engine.SaslListener;
import org.apache.qpid.proton.engine.Transport;

public class AmqpErrorHandler extends BaseHandler implements SaslListener {
    private final String queueName;

    private final String containerId;

    public AmqpErrorHandler(String queueName, String containerId) {
        this.queueName = queueName;
        this.containerId = containerId;
    }

    @Override
    public void onTransportError(Event e) {
        LogUtil.error("Transport error. queue={}, containerId={}, msg={}", queueName, containerId,
                e.getTransport().getRemoteCondition().toString());
    }

    @Override
    public void onLinkRemoteClose(Event e) {
        LogUtil.error("Link remote close. queue={}, containerId={}, msg={}", queueName, containerId,
                e.getLink().getRemoteCondition().toString());
    }

    @Override
    public void onLinkRemoteDetach(Event e) {
        LogUtil.error("Link remote detach. queue={}, containerId={}, msg={}", queueName, containerId,
                e.getLink().getRemoteCondition().toString());
    }

    @Override
    public void onSessionRemoteClose(Event e) {
        LogUtil.error("Session remote close. queue={}, containerId={}, msg={}", queueName, containerId,
                e.getSession().getRemoteCondition().toString());
    }

    @Override
    public void onConnectionRemoteClose(Event e) {
        LogUtil.error("Connection remote close. queue={}, containerId={}, msg={}", queueName, containerId,
                e.getConnection().getRemoteCondition().toString());
    }

    @Override
    public void onConnectionUnbound(Event e) {
        LogUtil.error("Connection unbound. queue={}, containerId={}, connectMsg={}",
                queueName, containerId, e.getConnection().getRemoteCondition().toString());
    }

    @Override
    public void onSaslMechanisms(Sasl sasl, Transport transport) {

    }

    @Override
    public void onSaslInit(Sasl sasl, Transport transport) {

    }

    @Override
    public void onSaslChallenge(Sasl sasl, Transport transport) {

    }

    @Override
    public void onSaslResponse(Sasl sasl, Transport transport) {

    }

    @Override
    public void onSaslOutcome(Sasl sasl, Transport transport) {

    }
}
