package com.huaweicloud.sdk.iot.amqpdemo.handler;

import com.huaweicloud.sdk.iot.amqpdemo.LogUtil;
import com.huaweicloud.sdk.iot.amqpdemo.client.AmqpContainer;

import org.apache.qpid.proton.engine.Transport;
import org.apache.qpid.proton.engine.TransportException;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

public class ChannelHandler {
    private SocketChannel socket;

    private SelectionKey key;

    private final Transport transport;

    private final String endpointStr;

    public ChannelHandler(AmqpContainer container) {
        this.transport = container.getTransport();
        this.endpointStr = container.getOptions().getEndpointStr();

    }

    public void init(SocketChannel socket, SelectionKey key) {
        this.socket = socket;
        this.key = key;
    }

    private boolean update() {
        if (socket.isConnected()) {
            int c = transport.capacity();
            int p = transport.pending();
            if (key.isValid()) {
                key.interestOps((c != 0 ? SelectionKey.OP_READ : 0) |
                        (p > 0 ? SelectionKey.OP_WRITE : 0));
            }
            return c < 0 && p < 0;
        } else {
            return false;
        }
    }

    public void handler() {
        if (!key.isValid()) {
            return;
        }

        try {
            if (key.isConnectable()) {
                socket.finishConnect();
                LogUtil.info("connect success: {}", socket);
            }

            if (key.isReadable()) {
                int c = transport.capacity();
                if (c > 0) {
                    ByteBuffer tail = transport.tail();
                    int n = socket.read(tail);
                    if (n > 0) {
                        try {
                            transport.process();
                        } catch (TransportException e) {
                            e.printStackTrace();
                        }
                    } else if (n < 0) {
                        transport.close_tail();
                    }
                }
            }

            if (key.isWritable()) {
                int p = transport.pending();
                if (p > 0) {
                    ByteBuffer head = transport.head();
                    int n = socket.write(head);
                    if (n > 0) {
                        transport.pop(n);
                    } else if (n < 0) {
                        transport.close_head();
                    }
                }
            }

            if (update()) {
                transport.unbind();
                LogUtil.error("Closed: {}, socket: {}", endpointStr, socket);
                socket.close();
            }
        } catch (IOException e) {
            transport.unbind();
            LogUtil.exceptionError(e, "Connect to {} failed, socket: {}", endpointStr, socket);
            try {
                socket.close();
            } catch (IOException e2) {
                throw new RuntimeException(e2);
            }
        }

    }
}
