package com.huaweicloud.sdk.iot.amqpdemo.handler;


import com.huaweicloud.sdk.iot.amqpdemo.client.AmqpContainer;

import org.apache.qpid.proton.Proton;
import org.apache.qpid.proton.codec.ReadableBuffer;
import org.apache.qpid.proton.engine.BaseHandler;
import org.apache.qpid.proton.engine.Delivery;
import org.apache.qpid.proton.engine.Event;
import org.apache.qpid.proton.engine.Receiver;
import org.apache.qpid.proton.message.Message;

public class DeliveryHandler extends BaseHandler {
    private final MessageListener messageListener;

    private final AmqpContainer amqpContainer;

    public DeliveryHandler(AmqpContainer amqpContainer, MessageListener messageListener) {
        this.amqpContainer = amqpContainer;
        this.messageListener = messageListener;
    }

    @Override
    public void onDelivery(Event evt) {
        Delivery dlv = evt.getDelivery();
        if (dlv.getLink() instanceof Receiver) {
            Receiver receiver = (Receiver) dlv.getLink();
            if (!dlv.isPartial()) {
                ReadableBuffer readableBuffer = receiver.recv();
                receiver.advance();
                Message msg = Proton.message();
                msg.decode(readableBuffer);
                if (messageListener != null) {
                    messageListener.onMessage(dlv, msg);
                }
                if (amqpContainer.getOptions().isAutoAcknowledge() && !dlv.isSettled()) {
                    dlv.settle();
                }
            }
        }
    }
}
